#!/bin/bash
#
# This script can be used to compile all the Samples using the currently
# installed (as reported by pkg-config) Ogre files. This is especially
# useful to test the Debian packages that do not include any Sample due
# to licensing problems.
#
# This script will attempt to download the source tarball in case it's not
# found in the current directory, unpack it, and compile the Samples.

OGRESRC=ogre-linux_osx-v1-4-5.tar.bz2
OGRESUM=efce9ce9e2b0e2d593e45479210bd4af

compile () {
    pushd >/dev/null $1/src
    out=Common/bin/$(basename $1)
    echo -n "Compiling Samples/$out ... "
    test -x ../../$out || \
        g++ -o ../../$out *.cpp -I../../Common/include -I../include \
            $(pkg-config --cflags --libs $2) $3
    echo "done"
    popd >/dev/null
}

check_deb () {
    if ! $(dpkg -l $1 2>&1 | grep >/dev/null 2>&1 ii) ; then
        echo "Error: package $1 is missing; please install it and re-run this script"
        exit 1
    fi
}

# Some basic checks for the build environment
check_deb libois-dev
check_deb libogre-dev

if [ ! -f $OGRESRC ]; then
    wget http://downloads.sourceforge.net/ogre/$OGRESRC
fi

CHECKSUM=$(md5sum $OGRESRC | cut -d ' ' -f 1)
if [ $OGRESUM != $CHECKSUM ]; then
    echo "Error: checksum verification failed:"
    echo "  computed checksum is  $CHECKSUM"
    echo "  expected checksum was $OGRESUM"
    exit 1
else
    echo "Checksum verified: $CHECKSUM"
fi

# Unpack the tarball
if [ ! -d ogrenew ]; then
	echo "Extracting $OGRESRC to ogrenew"
	tar -xjf $OGRESRC
else
	echo "Using existing ogrenew directory (remove it and re-run this script if you want to start from scratch)"
fi

# Link to the correct plugins.cfg for this system.
rm -f ogrenew/Samples/Common/bin/plugins.cfg
ln -s /etc/OGRE/plugins.cfg ogrenew/Samples/Common/bin/plugins.cfg

# Go into ogre samples directory and compile everything
cd ogrenew

if [ -z "$1" ] ; then
    SAMPLES="$(find Samples -maxdepth 1 -type d)"
else
    for name in $* ; do
        SAMPLES="$SAMPLES Samples/$name"
    done
fi

for sample in $SAMPLES ; do
    case $(basename $sample) in
    
      CVS|Samples|Common|Media)
        # nothing to do
      ;;
      
      GTKDemo|HDRDemo)
        # simply don't build
      ;;
	
      Gui)
        # compile $sample "OGRE CEGUI" -lCEGUIBase
      ;;
    
      Shadows|Compositor|FacialAnimation|OceanDemo|Instancing)
        # deactivated untill CEGUI is updated
      ;;

	BSP)
		# Don't build as it requires quake3 data files.
	;;

      *)
        compile $sample "OGRE OIS" 
      ;;
    
    esac
done

if ! $(dpkg -l ogre-plugins-cgprogrammanager 2>&1 | grep >/dev/null 2>&1 ii) ; then
	echo "The package ogre-plugins-cgprogrammanager is not installed (or is unavailable) on this system. This will significantly change the behavior of some demos."
fi

echo "Be sure to go into the ogrenew/Samples/Common/bin directory and run each demo from there."

# TODO: A proper "clean" option and implementation
# would also be useful to remove anything that was built.
